import React, { useEffect } from 'react';
import classnames from 'classnames';
import { suffixMetadata, parseMetadata } from '@highlight-ui/utils-commons';
import { Icon } from '@highlight-ui/icon';
import { Body } from '@highlight-ui/typography';
import { Inline } from '@highlight-ui/layout';
import { colorTokens, typographyColorTokens } from '@highlight-ui/tokens';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
function __rest(s, e) {
  var t = {};
  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
}
var styles = {
  "button": "BaseButton-module__cqxEoQOz__v12-2-0-r18-0",
  "label": "BaseButton-module__368a8XpF__v12-2-0-r18-0",
  "labelIcon": "BaseButton-module__ll4hx5mI__v12-2-0-r18-0",
  "labelIconContainer": "BaseButton-module__4CF-mqal__v12-2-0-r18-0",
  "loading": "BaseButton-module__AxaPQljF__v12-2-0-r18-0",
  "withIcon": "BaseButton-module__PAb94WZR__v12-2-0-r18-0",
  "iconButton": "BaseButton-module__w2FxOhsI__v12-2-0-r18-0",
  "hiddenButtonLabel": "BaseButton-module__d-kHuHzf__v12-2-0-r18-0",
  "loadingIcon": "BaseButton-module__SpYNEHQq__v12-2-0-r18-0",
  "block": "BaseButton-module__YKg1L4V-__v12-2-0-r18-0",
  "sr-only": "BaseButton-module__uHzk6SXO__v12-2-0-r18-0"
};
var BaseButton = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var _b, _c, _d;
  var block = _a.block,
    _e = _a.children,
    children = _e === void 0 ? null : _e,
    className = _a.className,
    _f = _a.buttonState,
    buttonState = _f === void 0 ? 'default' : _f,
    metadata = _a.metadata,
    _g = _a.onClick,
    onClick = _g === void 0 ? undefined : _g,
    _h = _a.type,
    type = _h === void 0 ? 'button' : _h,
    icon = _a.icon,
    _default = _a._default,
    _hover = _a._hover,
    _active = _a._active,
    _disabled = _a._disabled,
    _loading = _a._loading,
    remainingProps = __rest(_a, ["block", "children", "className", "buttonState", "metadata", "onClick", "type", "icon", "_default", "_hover", "_active", "_disabled", "_loading"]);
  var states = {
    default: _default,
    hover: _hover,
    active: _active,
    disabled: _disabled
  };
  var statesStyles = Object.entries(states).reduce(function (buttonStyles, _a) {
    var _b;
    var key = _a[0],
      state = _a[1];
    if (state) {
      return __assign(__assign({}, buttonStyles), (_b = {}, _b["--".concat(key, "-background-color")] = state.backgroundColor ? colorTokens[state.backgroundColor] : 'none', _b["--".concat(key, "-label-color")] = __assign(__assign({}, typographyColorTokens), colorTokens)[state.color], _b));
    }
    return buttonStyles;
  }, {});
  var containerClass = classnames(styles.button, (_b = {}, _b[styles.block] = block, _b[styles.loading] = buttonState === 'loading', _b[styles.withIcon] = icon && children, _b[styles.iconButton] = icon && !children, _b), className);
  var focusAwareOnClick = function (event) {
    event.currentTarget.focus();
    if (onClick) onClick(event);
  };
  var iconElement = icon && /*#__PURE__*/React.createElement("span", {
    className: classnames(styles.labelIconContainer, (_c = {}, _c[styles.hiddenButtonLabel] = buttonState === 'loading', _c))
  }, /*#__PURE__*/React.createElement(Icon, {
    name: icon,
    size: 14,
    type: "solid",
    "aria-hidden": true,
    className: styles.labelIcon,
    metadata: suffixMetadata(metadata, "".concat(icon, "-icon"))
  }));
  return /*#__PURE__*/React.createElement("button", __assign({}, remainingProps, parseMetadata(metadata), {
    className: containerClass,
    disabled: ['loading', 'disabled'].includes(buttonState),
    onClick: focusAwareOnClick,
    style: statesStyles,
    /* eslint-disable react/button-has-type */
    type: type,
    "aria-disabled": ['loading', 'disabled'].includes(buttonState),
    ref: ref
  }), !children && iconElement, children && /*#__PURE__*/React.createElement(Inline, {
    space: "spacing-1",
    alignVertical: "center",
    align: "center",
    className: classnames(styles.label, (_d = {}, _d[styles.hiddenButtonLabel] = buttonState === 'loading', _d))
  }, iconElement, /*#__PURE__*/React.createElement(Body, {
    variant: "strong-base"
  }, children)), buttonState === 'loading' && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("span", {
    "data-test-id": "hidden-span",
    className: "sr-only"
  }, "loading"), /*#__PURE__*/React.createElement(Icon, {
    spin: true,
    size: 16,
    type: "solid",
    name: "spinner-third",
    "aria-hidden": true,
    color: _loading === null || _loading === void 0 ? void 0 : _loading.color,
    className: styles.loadingIcon,
    metadata: suffixMetadata(metadata, 'loading-icon')
  })));
});
var buttonVariantToState = {
  default: {
    _default: {
      backgroundColor: 'action-secondary-default',
      color: 'text-default'
    },
    _active: {
      backgroundColor: 'action-secondary-active',
      color: 'text-default'
    },
    _disabled: {
      backgroundColor: 'action-secondary-disabled',
      color: 'text-disabled'
    },
    _hover: {
      backgroundColor: 'action-secondary-hover',
      color: 'text-default'
    },
    _loading: {
      color: 'icon-highlight' // TODO: apply branded.Dark token (not an icon color)
    }
  },

  emphasized: {
    _default: {
      backgroundColor: 'action-primary-default',
      color: 'text-on-primary'
    },
    _active: {
      backgroundColor: 'action-primary-active',
      color: 'text-on-emphasis'
    },
    _disabled: {
      backgroundColor: 'action-primary-disabled',
      color: 'text-on-primary'
    },
    _hover: {
      backgroundColor: 'action-primary-hover',
      color: 'text-on-emphasis'
    },
    _loading: {
      color: 'icon-on-primary'
    }
  },
  plain: {
    _default: {
      backgroundColor: null,
      color: 'action-primary-default'
    },
    _active: {
      backgroundColor: null,
      color: 'action-primary-active'
    },
    _disabled: {
      backgroundColor: null,
      color: 'action-primary-disabled'
    },
    _hover: {
      backgroundColor: null,
      color: 'action-primary-hover'
    },
    _loading: {
      color: 'icon-highlight' // TODO: apply interactive brand default (not an icon color)
    }
  }
};

var iconButtonVariantToState = {
  default: {
    _default: {
      backgroundColor: 'action-secondary-default',
      color: 'icon-default'
    },
    _active: {
      backgroundColor: 'action-secondary-active',
      color: 'icon-default'
    },
    _disabled: {
      backgroundColor: 'action-secondary-disabled',
      color: 'icon-disabled'
    },
    _hover: {
      backgroundColor: 'action-secondary-hover',
      color: 'icon-default'
    },
    _loading: {
      color: 'icon-highlight'
    }
  },
  emphasized: {
    _default: {
      backgroundColor: 'action-primary-default',
      color: 'icon-on-emphasis'
    },
    _active: {
      backgroundColor: 'action-primary-active',
      color: 'icon-on-emphasis'
    },
    _disabled: {
      backgroundColor: 'action-primary-disabled',
      color: 'icon-on-primary'
    },
    _hover: {
      backgroundColor: 'action-primary-hover',
      color: 'icon-on-emphasis'
    },
    _loading: {
      color: 'icon-on-primary'
    }
  },
  plain: {
    _default: {
      backgroundColor: null,
      color: 'icon-default'
    },
    _hover: {
      backgroundColor: 'background-hover',
      color: 'icon-default'
    },
    _active: {
      backgroundColor: 'background-subdued-selected',
      color: 'icon-default'
    },
    _disabled: {
      backgroundColor: null,
      color: 'icon-disabled'
    },
    _loading: {
      color: 'icon-highlight' // TODO: apply interactive brand default (not an icon color)
    }
  }
};

var buttonCriticalToneState = {
  emphasized: {
    _default: {
      backgroundColor: 'action-critical-default',
      color: 'text-on-emphasis'
    },
    _active: {
      backgroundColor: 'action-critical-focus',
      color: 'text-on-emphasis'
    },
    _hover: {
      backgroundColor: 'action-critical-hover',
      color: 'text-on-emphasis'
    },
    _disabled: {
      backgroundColor: 'action-critical-disabled',
      color: 'text-on-action-critical'
    },
    _loading: {
      color: 'icon-on-primary'
    }
  },
  plain: {
    _default: {
      backgroundColor: null,
      color: 'action-critical-default'
    },
    _hover: {
      backgroundColor: null,
      color: 'action-critical-hover'
    },
    _active: {
      backgroundColor: null,
      color: 'action-critical-focus'
    },
    _disabled: {
      backgroundColor: null,
      color: 'action-critical-disabled'
    },
    _loading: {
      color: 'icon-critical' // TODO: apply action-critical-default (not an icon color)
    }
  },

  default: {
    _default: {
      backgroundColor: 'neutral-10',
      color: 'action-critical-default'
    },
    _active: {
      backgroundColor: 'neutral-10',
      color: 'action-critical-default'
    },
    _hover: {
      backgroundColor: 'neutral-20',
      color: 'action-critical-default'
    },
    _disabled: {
      backgroundColor: 'neutral-10',
      color: 'action-critical-disabled'
    },
    _loading: {
      color: 'icon-critical'
    }
  }
};
var buttonSuccessToneState = {
  emphasized: {
    _default: {
      backgroundColor: 'action-success-default',
      color: 'text-on-emphasis'
    },
    _active: {
      backgroundColor: 'action-success-active',
      color: 'text-on-emphasis'
    },
    _hover: {
      backgroundColor: 'action-success-hover',
      color: 'text-on-emphasis'
    },
    _disabled: {
      backgroundColor: 'action-success-disabled',
      color: 'text-on-action-success'
    },
    _loading: {
      color: 'icon-on-success'
    }
  }
};
var Button = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var _b = _a.variant,
    variant = _b === void 0 ? 'default' : _b,
    _c = _a.tone,
    tone = _c === void 0 ? 'default' : _c,
    children = _a.children,
    props = __rest(_a, ["variant", "tone", "children"]);
  useEffect(function () {
    if (typeof children !== 'string') {
      console.warn('The children property should be passed as string for the button label and will be required in a future release!');
    }
  }, [children]);
  return /*#__PURE__*/React.createElement(BaseButton, __assign({}, props, tone === 'critical' ? buttonCriticalToneState[variant] : buttonSuccessToneState.emphasized, tone === 'default' ? buttonVariantToState[variant] : {}, {
    ref: ref
  }), children);
});
Button.displayName = 'Button';
var IconButton = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var _b = _a.variant,
    variant = _b === void 0 ? 'default' : _b,
    _c = _a.buttonState,
    buttonState = _c === void 0 ? 'default' : _c,
    props = __rest(_a, ["variant", "buttonState"]);
  var stateProps = iconButtonVariantToState[variant];
  return /*#__PURE__*/React.createElement(BaseButton, __assign({}, props, stateProps, {
    ref: ref,
    buttonState: buttonState
  }));
});
IconButton.displayName = 'IconButton';
export { Button, IconButton };
//# sourceMappingURL=index.js.map
